<?php

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @link       https://www.biuro.lt
 * @since      1.0.0
 *
 * @package    Biuro_Contacts
 * @subpackage Biuro_Contacts/includes
 */

/**
 * The core plugin class.
 *
 * This is used to define internationalization, admin-specific hooks, and
 * public-facing site hooks.
 *
 * Also maintains the unique identifier of this plugin as well as the current
 * version of the plugin.
 *
 * @since      1.0.0
 * @package    Biuro_Contacts
 * @subpackage Biuro_Contacts/includes
 * @author     Biuro <info@biuro.lt>
 */
class Biuro_Contacts {

    /**
     * The loader that's responsible for maintaining and registering all hooks that power
     * the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      Biuro_Contacts_Loader    $loader    Maintains and registers all hooks for the plugin.
     */
    protected $loader;

    /**
     * The unique identifier of this plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $plugin_name    The string used to uniquely identify this plugin.
     */
    protected $plugin_name;

    /**
     * The current version of the plugin.
     *
     * @since    1.0.0
     * @access   protected
     * @var      string    $version    The current version of the plugin.
     */
    protected $version;

    /**
     * Define the core functionality of the plugin.
     *
     * Set the plugin name and the plugin version that can be used throughout the plugin.
     * Load the dependencies, define the locale, and set the hooks for the admin area and
     * the public-facing side of the site.
     *
     * @since    1.0.0
     */
    public function __construct() {
        if ( defined( 'BIURO_CONTACTS_VERSION' ) ) {
            $this->version = BIURO_CONTACTS_VERSION;
        } else {
            $this->version = '1.0.0';
        }
        $this->plugin_name = 'biuro-contacts';

        $this->load_dependencies();
        $this->set_locale();
        $this->define_admin_hooks();
        $this->define_public_hooks();

    }

    /**
     * Load the required dependencies for this plugin.
     *
     * Include the following files that make up the plugin:
     *
     * - Biuro_Contacts_Loader. Orchestrates the hooks of the plugin.
     * - Biuro_Contacts_i18n. Defines internationalization functionality.
     * - Biuro_Contacts_Admin. Defines all hooks for the admin area.
     * - Biuro_Contacts_Public. Defines all hooks for the public side of the site.
     *
     * Create an instance of the loader which will be used to register the hooks
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function load_dependencies() {

        /**
         * The class responsible for orchestrating the actions and filters of the
         * core plugin.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-biuro-contacts-loader.php';

        /**
         * The class responsible for defining internationalization functionality
         * of the plugin.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'includes/class-biuro-contacts-i18n.php';

        /**
         * The class responsible for defining all actions that occur in the admin area.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'admin/class-biuro-contacts-admin.php';

        /**
         * The class responsible for defining all actions that occur in the public-facing
         * side of the site.
         */
        require_once plugin_dir_path( dirname( __FILE__ ) ) . 'public/class-biuro-contacts-public.php';

        $this->loader = new Biuro_Contacts_Loader();

    }

    /**
     * Define the locale for this plugin for internationalization.
     *
     * Uses the Biuro_Contacts_i18n class in order to set the domain and to register the hook
     * with WordPress.
     *
     * @since    1.0.0
     * @access   private
     */
    private function set_locale() {

        $plugin_i18n = new Biuro_Contacts_i18n();

        $this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

    }

    /**
     * Register all of the hooks related to the admin area functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_admin_hooks() {

        $plugin_admin = new Biuro_Contacts_Admin( $this->get_plugin_name(), $this->get_version() );

        $this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_styles' );
        $this->loader->add_action( 'admin_enqueue_scripts', $plugin_admin, 'enqueue_scripts' );
    }

    /**
     * Register all of the hooks related to the public-facing functionality
     * of the plugin.
     *
     * @since    1.0.0
     * @access   private
     */
    private function define_public_hooks() {

        $plugin_public = new Biuro_Contacts_Public( $this->get_plugin_name(), $this->get_version() );

        $this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
        $this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );

        $this->loader->add_action('admin_post_employees_post', $plugin_public, 'employees_post');
        $this->loader->add_action('admin_post_nopriv_employees_post', $plugin_public, 'employees_post');

        $this->loader->add_action('admin_post_employers_post', $plugin_public, 'employers_post');
        $this->loader->add_action('admin_post_nopriv_employers_post', $plugin_public, 'employers_post');

        $this->loader->add_action('admin_post_position_post', $plugin_public, 'position_post');
        $this->loader->add_action('admin_post_nopriv_position_post', $plugin_public, 'position_post');

        $this->loader->add_action('admin_post_recommend_post', $plugin_public, 'recommend_post');
        $this->loader->add_action('admin_post_nopriv_recommend_post', $plugin_public, 'recommend_post');

        // $this->loader->add_action( 'init', $plugin_public, 'register_shortcodes' );

        /**
         * Register shortcode via loader
         *
         * Use: [short-code-name args]
         *
         * @link https://github.com/DevinVinson/WordPress-Plugin-Boilerplate/issues/262
         */
        $this->loader->add_shortcode( "biuro-contacts--employees", $plugin_public, "employees_form", $priority = 10, $accepted_args = 2 );
        $this->loader->add_shortcode( "biuro-contacts--employers", $plugin_public, "employers_form", $priority = 10, $accepted_args = 2 );
        $this->loader->add_shortcode( "biuro-contacts--position", $plugin_public, "position_form", $priority = 10, $accepted_args = 2 );
        $this->loader->add_shortcode( "biuro-contacts--recommend", $plugin_public, "recommend_form", $priority = 10, $accepted_args = 2 );
    }

    /**
     * Run the loader to execute all of the hooks with WordPress.
     *
     * @since    1.0.0
     */
    public function run() {
        $this->loader->run();
    }

    /**
     * The name of the plugin used to uniquely identify it within the context of
     * WordPress and to define internationalization functionality.
     *
     * @since     1.0.0
     * @return    string    The name of the plugin.
     */
    public function get_plugin_name() {
        return $this->plugin_name;
    }

    /**
     * The reference to the class that orchestrates the hooks with the plugin.
     *
     * @since     1.0.0
     * @return    Biuro_Contacts_Loader    Orchestrates the hooks of the plugin.
     */
    public function get_loader() {
        return $this->loader;
    }

    /**
     * Retrieve the version number of the plugin.
     *
     * @since     1.0.0
     * @return    string    The version number of the plugin.
     */
    public function get_version() {
        return $this->version;
    }

}
