// https://developers-dot-devsite-v2-prod.appspot.com/maps/documentation/javascript/examples/overlay-popup
function createPopupClass () {
    function Popup (position, city, pos = 'top') {
        this.position = position;

        var content = document.createElement('div');
        content.innerHTML = city || '';

        content.classList.add('popup-bubble');
        content.classList.add('popup-bubble--' + pos);

        // This zero-height div is positioned at the bottom of the bubble.
        var bubbleAnchor = document.createElement('div');
        bubbleAnchor.classList.add('popup-bubble-anchor');
        bubbleAnchor.appendChild(content);

        // This zero-height div is positioned at the bottom of the tip.
        this.containerDiv = document.createElement('div');
        this.containerDiv.classList.add('popup-container');
        this.containerDiv.appendChild(bubbleAnchor);

        // Optionally stop clicks, etc., from bubbling up to the map.
        window.google.maps.OverlayView.preventMapHitsAndGesturesFrom(this.containerDiv);
    }
    // ES5 magic to extend google.maps.OverlayView.
    Popup.prototype = Object.create(window.google.maps.OverlayView.prototype);

    /** Called when the popup is added to the map. */
    Popup.prototype.onAdd = function () {
        this.getPanes().floatPane.appendChild(this.containerDiv);
    };

    /** Called when the popup is removed from the map. */
    Popup.prototype.onRemove = function () {
        if (this.containerDiv.parentElement) {
            this.containerDiv.parentElement.removeChild(this.containerDiv);
        }
    };

    /** Called each frame when the popup needs to draw itself. */
    Popup.prototype.draw = function () {
        var divPosition = this.getProjection().fromLatLngToDivPixel(this.position);

        // Hide the popup when it is far out of view.
        var display =
        Math.abs(divPosition.x) < 4000 && Math.abs(divPosition.y) < 4000 ? 'block' : 'none';

        if (display === 'block') {
            this.containerDiv.style.left = divPosition.x + 'px';
            this.containerDiv.style.top = divPosition.y + 'px';
        }
        if (this.containerDiv.style.display !== display) {
            this.containerDiv.style.display = display;
        }
    };

    return Popup;
}

var info;

function setCitiesMarkers (map, positions) {
    const Popup = createPopupClass();
    const bounds = new window.google.maps.LatLngBounds();

    let popup;

    for (var i = 0; i < positions.length; i++) {
        const position = positions[i];
        const pos = new window.google.maps.LatLng(position.lat, position.lng);

        popup = new Popup(pos, position.title, position.pos);
        popup.setMap(map);

        bounds.extend(pos);
    }

    map.fitBounds(bounds);
}

function setMarkers (map, positions, focus) {
    var bounds = new window.google.maps.LatLngBounds();

    for (var i = 0; i < positions.length; i++) {
        const position = positions[i];
        const pos = new window.google.maps.LatLng(position.lat, position.lng);

        var marker = new window.google.maps.Marker({
            position: pos,
            map: map,
            icon: '/wp-content/themes/biuro/i/ico--map-pin.svg',
            title: position.title || ''
        });

        if (info) {
            info.close();
        }

        bounds.extend(pos);

        if (position.content) {
            marker.content = position.content;

            window.google.maps.event.addListener(marker, 'click', function () {
                if (info) {
                    info.close();
                }

                info = new window.google.maps.InfoWindow({
                    content: this.content
                });

                info.open(map, this);
            });
        }

        if (focus) {
            if (window.innerWidth < 960) {
                window.scrollTo(0, 0);
            }
            new google.maps.event.trigger( marker, 'click' );
        }
    }

    map.fitBounds(bounds);

    if (window.innerWidth > 1020) {
        map.panBy(250, 0);
    } else if (window.innerWidth > 959) {
        map.panBy(180, 0);
    }
}

function setGoogleMap (node) {
    const mapStyles = [
        {
            featureType: 'all',
            elementType: 'all',
            stylers: [{ saturation: -92 }, { lightness: -8 }, { hue: '#004ed4' }]
        },
        {
            featureType: 'water',
            elementType: 'all',
            stylers: [{ saturation: -95 }, { lightness: -25 }, { hue: '#004ed4' }]
        }
    ];

    var map = new window.google.maps.Map(node, {});

    var biuroMap = new window.google.maps.StyledMapType(mapStyles, { name: 'Biuro' });

    map.mapTypes.set('biuro', biuroMap);

    map.setMapTypeId('biuro');

    window.google.maps.event.addListenerOnce(map, 'bounds_changed', function () {
        if (this.getZoom() > 15) {
            this.setZoom(14);
        }
    });

    return map;
}

function initCitiesMap (node, cities) {
    if (!window.google) {
        setTimeout(() => { initCitiesMap(node, cities); }, 250);
        return;
    }

    var map = setGoogleMap(node);

    setCitiesMarkers(map, cities);
}

function initRegionsMap (node) {
    if (!window.google) {
        setTimeout(() => { initRegionsMap(node); }, 250);
        return;
    }

    var map = setGoogleMap(node);

    const regions = [
        { title: 'Vilnius', lat: 54.687157, lng: 25.279652 },
        { title: 'Rīga', lat: 56.946285, lng: 24.105078 },
        { title: 'Tallinn', lat: 59.436962, lng: 24.753574 }
    ];

    setMarkers(map, regions);
}

function initDivisionsMap (node, data) {
    if (!window.google) {
        setTimeout(() => { initDivisionsMap(node, data); }, 250);
        return;
    }

    var map = setGoogleMap(node);

    let divisions = [];

    Object.keys(data).forEach((key) => {
        const division = data[key];
        if (key.substr(0, 4) === 'city') {
            divisions = divisions.concat(division);
        }
    });

    document.querySelectorAll('.js-division').forEach((node) => {
        node.addEventListener('click', () => {
            const ID = (node && node.dataset.id) ? node.dataset.id : '';

            if (data[ID]) {
                setMarkers(map, data[ID].filter((d) => { return d.lat && d.lng; }), true);
            } else {
                setMarkers(map, divisions.filter((d) => { return d.lat && d.lng; }));
            }
        });
    });

    setMarkers(map, divisions.filter((d) => { return d.lat && d.lng; }));
}

const inititateMap = () => {
    const division = document.getElementById('js-map--divisions');

    if (division) {
        fetch('/wp-json/api/v1/divisions?langID=' + division.dataset.id)
            .then((t) => t.json())
            .then((data) => {
                initDivisionsMap(division, data);
            });
    }

    const city = document.getElementById('js-map--cities');

    if (city) {
        fetch('/wp-content/themes/biuro/json/' + city.dataset.source + '.json')
            .then((t) => t.json())
            .then((data) => {
                initCitiesMap(city, data);
            });
    }

    const region = document.getElementById('js-map--regions');

    if (region) {
        initRegionsMap(region);
    }
};

export default inititateMap;
